APP_COMPONENT = "APP_COMPONENT"
APP_COMPONENT_TYPE = "APP_COMPONENT_TYPE"

APP_CONTROLLER = "APP_CONTROLLER"
APP_CONTROLLER_METHOD = "APP_CONTROLLER_METHOD"
APP_METHOD_MIDDLEWARE = "APP_METHOD_MIDDLEWARE"
APP_MIDDLEWARE_TARGET = "APP_MIDDLEWARE_TARGET"
